#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@File Name  : app.py
@Author     : LeeCQ
@Date-Time  : 2024/9/15 22:55
"""
import typer

from alist_sdk.cmd.base import cnf
from alist_sdk.cmd.fs import fs
from alist_sdk.cmd.admin import admin
from alist_sdk.cmd.auth import auth


app = typer.Typer()

app.add_typer(auth, name="auth")
app.add_typer(fs, name="fs")
app.add_typer(admin, name="admin")


@app.command("version")
def version():
    """
    Show the version of alist-sdk.
    """
    from alist_sdk import __version__

    typer.echo(f"alist-sdk version: {__version__}")


@app.command("server-version")
def server_version(server: str = None):
    """
    Show the version of alist-server.
    """
    from alist_sdk import Client

    if server and server not in cnf.auth_data:
        return typer.echo("尚未登录到该服务器，请先登录 [cmd: alist-cli auth login].")

    if not server:
        server = cnf.auth_data.keys()
    else:
        server = [server]

    typer.echo("Server Version:")
    for s in server:
        client = cnf.get_client(s)
        typer.echo(f"{s}: {client.service_version}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@File Name  : __init__.py.py
@Author     : LeeCQ
@Date-Time  : 2024/9/15 22:23
"""

# from .fs import *
# from .admin import *
# from .base import *



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@File Name  : auth.py
@Author     : LeeCQ
@Date-Time  : 2024/9/16 13:19
"""
import typer

from alist_sdk.cmd.base import cnf

auth = typer.Typer(name="auth", help="Authentication commands.")


@auth.command("login")
def login(host: str, username: str, password: str):
    """登录, 创建或添加一个更新的token到本地"""
    cnf.add_auth(host, username=username, password=password)


@auth.command("logout")
def logout(host: str):
    """登出, 删除本地的token"""
    cnf.remove_auth(host)


@auth.command("list")
def list_auth(token: bool = typer.Option(False, "--token", "-t", help="显示token")):
    """列出所有已登录的host"""
    if token:
        las = "\n".join(f"{host}: {_t}" for host, _t in cnf.auth_data.items())
    else:
        las = "\n".join(host for host in cnf.auth_data.keys())
    if not las:
        typer.echo("未登录到任何AlistServer")
        return
    typer.echo(f"已登录到的AlistServer:\n{las}")


@auth.command("refresh")
def refresh_token(host: str = typer.Argument(None, help="更新指定host的token")):
    """刷新token"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@File Name  : base.py
@Author     : LeeCQ
@Date-Time  : 2024/9/15 22:47
"""
import hashlib
import time
from pathlib import Path

import typer
from pydantic import BaseModel

from alist_sdk import Client, login_server, AlistPath

text_types = "txt,htm,html,xml,java,properties,sql,js,md,json,conf,ini,vue,php,py,bat,gitignore,yml,go,sh,c,cpp,h,hpp,tsx,vtt,srt,ass,rs,lrc,yaml"

CMD_BASE_PATH = ""
CONFIG_FILE_PATH = Path.home().joinpath(".config", "alist_cli.json")


def beautify_size(byte_size: float):
    if byte_size < 1024:
        return f"{byte_size:.2f}B"
    byte_size /= 1024
    if byte_size < 1024:
        return f"{byte_size:.2f}KB"
    byte_size /= 1024
    if byte_size < 1024:
        return f"{byte_size:.2f}MB"
    byte_size /= 1024
    return f"{byte_size:.2f}GB"


def xor(s: str, key: str) -> str:
    """
    字符串的亦或加密/解密
    :param s: 待加密/解密的字符串
    :param key: 加密/解密密钥
    :return: 加密/解密后的字符串
    """
    key = hashlib.md5(key.encode("utf-8")).hexdigest()
    return "".join(
        [chr(ord(c) ^ ord(k)) for c, k in zip(s, key * (len(s) // len(key) + 1))]
    )


class Auth(BaseModel):
    host: str
    token: str
    username: str
    password: str
    last_login: int = 0


class CmdConfig(BaseModel):
    auth_data: dict[str, Auth] = {}
    base_path: dict[str, str] = {}

    @classmethod
    def load_config(cls):
        """加载配置"""

        if not CONFIG_FILE_PATH.exists():
            return cls()
        try:
            typer.echo(f"load config from {CONFIG_FILE_PATH}")
            return cls.model_validate_json(
                xor(
                    CONFIG_FILE_PATH.read_bytes().decode("utf-8"),
                    CONFIG_FILE_PATH.as_posix(),
                )
            )
        except Exception as e:
            typer.echo(f"load config failed, {e}")
            wait_info = (
                f"Do you want to remove the config file[{CONFIG_FILE_PATH}]? (y/n)"
            )
            if input(wait_info).lower() == "y":
                CONFIG_FILE_PATH.unlink()
                typer.echo(f"remove config file success, {CONFIG_FILE_PATH}")
            exit(1)

    def save_config(self):
        """保存配置"""
        CONFIG_FILE_PATH.parent.mkdir(parents=True, exist_ok=True)

        print(f"save config to {self.model_dump_json()}")
        CONFIG_FILE_PATH.write_bytes(
            xor(self.model_dump_json(), CONFIG_FILE_PATH.as_posix()).encode("utf-8"),
        )

    def add_auth(
        self,
        host: str,
        username: str = None,
        password: str = None,
        token: str = None,
    ):
        host = host.strip("/")
        if not token:
            try:
                _c = Client(host, username=username, password=password)
                if _c.login_username != username:
                    typer.echo(f"login failed, username: {_c.login_username}")
                    return
                token = _c.get_token()
            except Exception as e:
                typer.echo(f"login failed, {e}")
                return

        self.auth_data[host] = Auth(
            host=host,
            token=token,
            username=username,
            password=password,
            last_login=int(time.time()),
        )
        self.save_config()
        typer.echo(f"login success, token: {self.auth_data[host].token}")

    def remove_auth(self, host: str):
        host = host.strip("/")
        if host not in self.auth_data:
            typer.echo(f"host {host} not found in auth data")
            return
        del self.auth_data[host]
        self.save_config()
        typer.echo(f"logout success, host: {host}")

    def get_client(self, host: str):
        host = host.strip("/")
        if host not in self.auth_data:
            raise ValueError(f"host {host} not found in auth data")
        t_info = self.auth_data[host]
        if int(time.time()) - t_info.last_login > 3600 * 24:
            self.add_auth(host, t_info.username, t_info.password)
        return login_server(host, token=self.auth_data[host].token)

    def set_base_path(self, base_path: str, name: str = ""):
        self.base_path[name] = base_path
        self.save_config()
        typer.echo(f"set base path success, base path: {self.base_path}")

    def remove_base_path(self, name: str):
        if name not in self.base_path:
            typer.echo(f"base path {name} not found in config")
            return
        del self.base_path[name]
        self.save_config()
        typer.echo(f"remove base path success, base path: {self.base_path}")

    def get_base_path(self, name: str = ""):
        if name not in self.base_path:
            typer.echo(f"base path {name} not found in config", err=True)
            exit(1)
        return self.base_path[name]


cnf = CmdConfig.load_config()


class CmdPath:
    def __new__(cls, *args, **kwargs) -> AlistPath | Path:
        """
        //base_path/name/file.txt -- 有名称的基于base_path的路径
        ///name/file.txt          -- 无名称的基于base_path的路径
        name/file.txt             -- 本地相对路径
        /name/file.txt            -- 本地绝对路径
        """
        if args[0].startswith("//"):
            name, rp = args[0][2:].split("/", 1)
            return AlistPath(cnf.get_base_path(name), rp, *args[1:], **kwargs)

        return Path(*args, **kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@File Name  : fs.py
@Author     : LeeCQ
@Date-Time  : 2024/9/15 22:47
"""
import subprocess
from pathlib import Path

import typer

from alist_sdk import AlistPath
from alist_sdk.cmd.base import beautify_size, text_types

fs = typer.Typer(name="fs", help="文件系统相关操作")


@fs.command("ls")
def ls(
    path: str,
):
    """列出文件系统"""
    path = AlistPath(path)
    ls = path.iterdir() if path.is_dir() else [path]
    # type, size, modify_time, name
    total_size = 0
    for p in ls:
        total_size += p.stat().size
        p_type = "dir" if p.is_dir() else "file"
        typer.echo(
            f"{p_type:<8}"
            f"{beautify_size(p.stat().size):<10}"
            f"{p.stat().modified.strftime('%Y-%m-%d %H:%M:%S'):<20} {p.name}"
        )
    typer.echo(f"total: {beautify_size(total_size)}")


@fs.command("print")
def fs_read(path: str):
    """读取文件内容"""
    path = AlistPath(path)
    _text_types = (
        {i.key: i.value for i in path.client.admin_setting_list(3).data}
        .get("text_types", text_types)
        .split(",")
    )
    if path.suffix not in _text_types:
        typer.echo(f"{path.suffix} 文件类型不支持直接读取")
    with path.read_bytes() as f:
        typer.echo(f"Read from {path.as_uri()}\n" + "=" * 50)
        typer.echo("\n" + f.read())


@fs.command("mkdir")
def mkdir(
    path: str = typer.Argument(..., help="父目录路径"),
    name: str = typer.Argument("", help="目录名称"),
    parents: bool = typer.Option(False, help="是否创建父目录"),
    exist_ok: bool = typer.Option(False, help="是否忽略已存在的目录"),
):
    """创建目录"""
    try:
        AlistPath(path).joinpath(name).mkdir(parents=parents, exist_ok=exist_ok)
    except Exception as e:
        typer.echo(f"mkdir error: {e}")


@fs.command("rm")
def rm(
    path: str,
    recursive: bool = typer.Option(False, help="是否递归删除"),
    force: bool = typer.Option(False, help="是否强制删除"),
):
    """删除文件或目录"""
    try:
        AlistPath(path).unlink(missing_ok=True)
    except Exception as e:
        typer.echo(f"rm error: {e}")


@fs.command("cp")
def cp(
    src: str,
    dst: str,
    recursive: bool = typer.Option(False, "-r", help="是否递归复制"),
):
    """复制文件 递归
    alist  - alist
    alist - local
    local - alist
    """


@fs.command("download")
def download(
    src: str,
    dst: str,
    recursive: bool = typer.Option(False, "-r", help="是否递归下载"),
):
    """下载文件"""

    def down_file(src_path, dst_path):
        down_link = AlistPath(src).as_download_uri()
        if Path(dst_path).exists():
            typer.echo(f"{dst_path} 已存在，跳过")
            return
        if not Path(dst_path).parent.exists():
            Path(dst_path).parent.mkdir(parents=True)
        typer.echo(f"downloading {src_path} to {dst_path}")
        subprocess.run(["curl", "-o", dst_path, down_link])  # TODO: 自定义UA

    src = AlistPath(src)
    dst = Path(dst)
    if src.is_dir():
        dst = dst.joinpath(src.name)
        dst.mkdir(parents=True, exist_ok=True)

    for p in src.iterdir():
        if p.is_dir() and recursive:
            download(str(p), str(dst.joinpath(p.name)))
        elif p.is_file():
            down_file(p, dst.joinpath(p.name))
        else:
            typer.echo(f"{p} 不是文件或目录，跳过")


@fs.command("upload")
def upload(
    src: str,
    dst: str,
    force: bool = typer.Option(False, help="是否覆盖已有文件"),
):
    """上传文件"""
    if not Path(src).exists() or not Path(src).is_file():
        typer.echo(f"{src} 不存在或不是文件")
        exit(1)
    if not AlistPath(dst).parent.exists():
        typer.echo(f"{AlistPath(dst).parent} 不存在")
        exit(1)
    if AlistPath(dst).exists() and not force:
        typer.echo(f"{dst} 已存在")
        exit(1)
    typer.echo(f"uploading {src} to {dst}")
    AlistPath(dst).write_bytes(Path(src))
